--
--


CREATE TABLE project_type (
    id integer PRIMARY KEY AUTOINCREMENT,
    code character varying(30),
    dir_naming_cls character varying(200),
    file_naming_cls character varying(200),
    code_naming_cls character varying(200),
    node_naming_cls character varying(200),
    sobject_mapping_cls character varying(200),
    s_status character varying(32),
    "type" character varying(100) NOT NULL,
    repo_handler_cls character varying(200),
    CONSTRAINT "project_type_code_idx" UNIQUE (code)
);



CREATE TABLE login_group (
    id integer PRIMARY KEY AUTOINCREMENT,
    code character varying(256),
    login_group character varying(100) NOT NULL,
    sub_groups text,
    access_rules text,
    redirect_url text,
    namespace character varying(255),
    description text,
    project_code text,
    s_status character varying(256),
    CONSTRAINT "login_group_code_idx" UNIQUE (code)
);



CREATE TABLE login_in_group (
    id integer PRIMARY KEY AUTOINCREMENT,
    "login" character varying(100) NOT NULL,
    login_group character varying(100) NOT NULL
);



CREATE TABLE ticket (
    id integer PRIMARY KEY AUTOINCREMENT,
    code character varying(256),
    ticket character varying(100) NOT NULL,
    "login" character varying(100),
    "timestamp" timestamp with time zone DEFAULT CURRENT_TIMESTAMP,
    expiry timestamp without time zone,
    category character varying(256),
    CONSTRAINT "ticket_code_idx" UNIQUE (code),
    CONSTRAINT "ticket_ticket_idx" UNIQUE (ticket)
);




CREATE TABLE pipeline (
    id integer PRIMARY KEY AUTOINCREMENT,
    code character varying(256),
    pipeline text,
    "timestamp" timestamp without time zone DEFAULT CURRENT_TIMESTAMP NOT NULL,
    search_type character varying(100),
    project_code character varying(30),
    description text,
    color varchar(256),
    s_status character varying(30),
    CONSTRAINT "pipeline_code_idx" UNIQUE (code)
);



-- FIXME: Is this needed anymore?
-- FIXME: notification_id will be deprecated in 4.0
CREATE TABLE group_notification (
    id integer PRIMARY KEY AUTOINCREMENT,
    login_group character varying(100) NOT NULL,
    notification_id integer NOT NULL
);
CREATE TABLE notification_login (
    id integer NOT NULL,
    notification_log_id integer,
    "login" character varying(256),
    "type" character varying(256),
    project_code character varying(256),
    "timestamp" timestamp without time zone DEFAULT CURRENT_TIMESTAMP
);



CREATE TABLE notification_log (
    id integer NOT NULL,
    project_code character varying(256),
    "login" character varying(256),
    command_cls character varying(256),
    subject text,
    message text,
    "timestamp" timestamp without time zone DEFAULT CURRENT_TIMESTAMP
);


CREATE TABLE snapshot (
    id integer PRIMARY KEY AUTOINCREMENT,
    search_type character varying(100) NOT NULL,
    search_id integer NOT NULL,
    column_name character varying(100) NOT NULL,
    snapshot text NOT NULL,
    description text,
    process varchar(256),
    "login" character varying(100) NOT NULL,
    "lock_login" character varying(100),
    "timestamp" timestamp without time zone DEFAULT CURRENT_TIMESTAMP NOT NULL,
    lock_date timestamp,
    context character varying(256),
    version integer,
    s_status character varying(30),
    snapshot_type character varying(30),
    code character varying(30),
    repo character varying(30),
    is_current boolean,
    label character varying(100),
    revision smallint,
    level_type character varying(256),
    level_id integer,
    metadata text,
    is_latest boolean,
    status character varying(256),
    project_code character varying(256),
    search_code character varying(256),
    is_synced boolean,
    CONSTRAINT "snapshot_code_idx" UNIQUE (code)
);

CREATE TABLE file (
    id integer PRIMARY KEY AUTOINCREMENT,
    file_name character varying(512) NOT NULL,
    search_type character varying(100) NOT NULL,
    search_id integer NOT NULL,
    "timestamp" timestamp without time zone DEFAULT CURRENT_TIMESTAMP NOT NULL,
    st_size bigint,
    file_range text,
    code character varying(30),
    snapshot_code character varying(30),
    project_code character varying(100),
    md5 character varying(32),
    checkin_dir text,
    source_path text,
    relative_dir text,
    "type" character varying(256),
    base_type character varying(256),
    CONSTRAINT "file_code_idx" UNIQUE (code)
);



-- NOTE: is this really needed anymore?  Check-in is dependent on it!
CREATE TABLE remote_repo (
    id integer PRIMARY KEY AUTOINCREMENT,
    code character varying(30),
    ip_address character varying(30),
    ip_mask character varying(30),
    repo_base_dir character varying(200),
    sandbox_base_dir character varying(200),
    "login" character varying(100),
    CONSTRAINT "remote_repo_code_idx" UNIQUE (code)
);




CREATE TABLE milestone (
    id integer PRIMARY KEY AUTOINCREMENT,
    code character varying(256),
    project_code character varying(256),
    description text,
    due_date timestamp without time zone,
    CONSTRAINT "milestone_code_idx" UNIQUE (code)
);




-- NOTE: file name is deleted
CREATE TABLE task (
    id integer PRIMARY KEY AUTOINCREMENT,
    -- file_name character varying(512) NOT NULL,
    assigned character varying(100),
    description text,
    status text,
    discussion text,
    bid_start_date timestamp without time zone,
    bid_end_date timestamp without time zone,
    bid_duration double precision,
    actual_start_date timestamp without time zone,
    actual_end_date timestamp without time zone,
    search_type character varying(100),
    search_id integer,
    "timestamp" timestamp without time zone DEFAULT CURRENT_TIMESTAMP,
    s_status character varying(30),
    priority smallint,
    process character varying(256),
    context character varying(256),
    milestone_code character varying(200),
    pipeline_code character varying(256),
    parent_id integer,
    sort_order smallint,
    depend_id integer,
    project_code character varying(100),
    supervisor character varying(100),
    code character varying(256),
    completion float,
    CONSTRAINT "task_code_idx" UNIQUE (code)
);


CREATE TABLE note (
    id integer PRIMARY KEY AUTOINCREMENT,
    code character varying(256),
    project_code character varying(30),
    search_type character varying(200),
    search_id integer,
    "login" character varying(30),
    context character varying(60),
    "timestamp" timestamp without time zone DEFAULT CURRENT_TIMESTAMP,
    note text,
    title character varying(1024),
    parent_id bigint,
    status character varying(256),
    label character varying(256),
    process character varying(60),
    "sort_order" integer,
    "access" character varying(256),
    CONSTRAINT "note_code_idx" UNIQUE (code)
);




CREATE TABLE pref_setting (
    id integer PRIMARY KEY AUTOINCREMENT,
    project_code character varying(256),
    "login" character varying(256),
    "key" character varying(256),
    value text,
    "timestamp" timestamp without time zone DEFAULT CURRENT_TIMESTAMP
);


CREATE TABLE wdg_settings (
    id integer PRIMARY KEY AUTOINCREMENT,
    "key" character varying(255) NOT NULL,
    "login" character varying(100) NOT NULL,
    data text,
    "timestamp" timestamp without time zone DEFAULT CURRENT_TIMESTAMP NOT NULL,
    project_code character varying(30)
);



CREATE TABLE clipboard (
    id integer PRIMARY KEY AUTOINCREMENT,
    project_code character varying(256),
    "login" character varying(256),
    search_type character varying(256),
    search_id integer,
    "timestamp" timestamp without time zone DEFAULT CURRENT_TIMESTAMP,
    category character varying(256)
);






CREATE TABLE exception_log (
    id integer PRIMARY KEY AUTOINCREMENT,
    "class" character varying(100),
    message text,
    stack_trace text,
    "login" character varying(100) NOT NULL,
    "timestamp" timestamp without time zone DEFAULT CURRENT_TIMESTAMP NOT NULL
);




CREATE TABLE transaction_log (
    id integer PRIMARY KEY AUTOINCREMENT,
    "transaction" text,
    "login" character varying(100) NOT NULL,
    "timestamp" timestamp without time zone DEFAULT CURRENT_TIMESTAMP NOT NULL,
    description text,
    command character varying(100),
    title text,
    "type" character varying(30),
    namespace character varying(100)
);




-- FIXME: is this really needed???
CREATE TABLE transaction_state (
    id integer PRIMARY KEY AUTOINCREMENT,
    ticket character varying(100),
    "timestamp" timestamp without time zone DEFAULT CURRENT_TIMESTAMP,
    data text
);




CREATE TABLE sobject_log (
    id integer PRIMARY KEY AUTOINCREMENT,
    search_type character varying(100) NOT NULL,
    search_id integer NOT NULL,
    data text,
    "login" character varying(100) NOT NULL,
    "timestamp" timestamp without time zone DEFAULT CURRENT_TIMESTAMP NOT NULL,
    transaction_log_id integer
);


CREATE TABLE status_log (
    id integer PRIMARY KEY AUTOINCREMENT,
    search_type character varying(256) NOT NULL,
    search_id integer NOT NULL,
    status text,
    "login" character varying(256) NOT NULL,
    "timestamp" timestamp without time zone DEFAULT CURRENT_TIMESTAMP NOT NULL,
    to_status character varying(256),
    from_status character varying(256),
    project_code character varying(256)
);

CREATE TABLE debug_log (
    id integer NOT NULL,
    category character varying(256),
    "level" character varying(256),
    message text,
    "timestamp" timestamp without time zone DEFAULT CURRENT_TIMESTAMP,
    "login" character varying(256),
    s_status character varying(30)
);



CREATE TABLE work_hour (
    id integer NOT NULL,
    code character varying(256),
    project_code character varying(256),
    description text,
    category character varying(256),
    "login" character varying(256),
    "day" timestamp without time zone,
    start_time timestamp without time zone,
    end_time timestamp without time zone,
    straight_time double precision,
    over_time double precision,
    search_type character varying(256),
    search_id integer,
    status character varying(256),
    CONSTRAINT "work_hour_code_idx" UNIQUE (code)
);



-- in upgrade
CREATE TABLE cache (
    id integer PRIMARY KEY AUTOINCREMENT,
    "key" varchar(256),
    mtime timestamp
);


CREATE TABLE sobject_list (
    id integer PRIMARY KEY AUTOINCREMENT,
    search_type varchar(256),
    search_id integer,
    keywords text,
    timestamp timestamp default CURRENT_TIMESTAMP,
    project_code varchar(256)
);




INSERT INTO project (code) VALUES ('unittest');



-- FIXME: are thse needed anymore?
-- INSERT INTO search_object (search_type, namespace, description, "database", table_name, class_name, title, "schema") VALUES (87, 'prod/naming', 'prod', 'Naming', '{project}', '{public}.naming', 'pyasm.biz.Naming', '', 'public');

-- INSERT INTO search_object (search_type, namespace, description, "database", table_name, class_name, title, "schema") VALUES (97, 'prod/custom_property', 'sthpw', 'Custom Property', '{project}', 'custom_property', 'pyasm.biz.CustomProperty', 'Custom Property', 'public');





INSERT INTO search_object (search_type, namespace, description, "database", table_name, class_name, title, "schema") VALUES ('sthpw/annotation', 'sthpw', 'Image Annotations', 'sthpw', 'annotation', 'pyasm.search.search.SObject', 'Image Annotations', 'public');
INSERT INTO search_object (search_type, namespace, description, "database", table_name, class_name, title, "schema") VALUES ('sthpw/retire_log', 'sthpw', 'Retire SObject log', 'sthpw', 'retire_log', 'pyasm.search.RetireLog', 'Retire SObject log', 'public');
INSERT INTO search_object (search_type, namespace, description, "database", table_name, class_name, title, "schema") VALUES ('sthpw/login_in_group', 'sthpw', 'Users in groups', 'sthpw', 'login_in_group', 'pyasm.security.LoginInGroup', 'Users in groups', 'public');
INSERT INTO search_object (search_type, namespace, description, "database", table_name, class_name, title, "schema") VALUES ('sthpw/exception_log', 'sthpw', 'Exception Log', 'sthpw', 'exception_log', 'pyasm.search.SObject', 'Exception Log', 'public');
INSERT INTO search_object (search_type, namespace, description, "database", table_name, class_name, title, "schema") VALUES ('sthpw/file_access', 'sthpw', 'File Access Log', 'sthpw', 'file_access', 'pyasm.biz.FileAccess', 'File Access Log', 'public');
INSERT INTO search_object (search_type, namespace, description, "database", table_name, class_name, title, "schema") VALUES ('sthpw/repo', 'sthpw', 'Repository List', 'sthpw', 'repo', 'pyasm.search.SObject', 'Repository List', NULL);
INSERT INTO search_object (search_type, namespace, description, "database", table_name, class_name, title, "schema") VALUES ('sthpw/queue', 'sthpw', 'Tactic Dispatcher', 'sthpw', 'queue', 'pyasm.search.SObject', 'Tactic Dispatcher', NULL);
INSERT INTO search_object (search_type, namespace, description, "database", table_name, class_name, title, "schema") VALUES ('sthpw/template', 'sthpw', 'SObject Templates', 'sthpw', 'template', 'pyasm.biz.Template', 'SObject Templates', 'public');
INSERT INTO search_object (search_type, namespace, description, "database", table_name, class_name, title, "schema") VALUES ('sthpw/project', 'sthpw', 'Projects', 'sthpw', 'project', 'pyasm.biz.Project', 'Projects', NULL);
INSERT INTO search_object (search_type, namespace, description, "database", table_name, class_name, title, "schema") VALUES ('sthpw/task', 'sthpw', 'User Tasks', 'sthpw', 'task', 'pyasm.biz.Task', 'User Tasks', NULL);
INSERT INTO search_object (search_type, namespace, description, "database", table_name, class_name, title, "schema") VALUES ('sthpw/sobject_config', 'sthpw', 'SObject Config Data', 'sthpw', 'sobject_config', 'pyasm.search.SObjectDbConfig', 'SObject Config Data', 'public');
INSERT INTO search_object (search_type, namespace, description, "database", table_name, class_name, title, "schema") VALUES ('sthpw/transaction_state', 'sthpw', 'XMLRPC State', 'sthpw', 'transaction_state', 'pyasm.search.TransactionState', 'transaction_state', 'public');
INSERT INTO search_object (search_type, namespace, description, "database", table_name, class_name, title, "schema") VALUES ('sthpw/command', 'sthpw', 'Commands in Tactic', 'sthpw', 'command', 'pyasm.biz.CommandSObj', 'Commands in Tactic', 'public');
INSERT INTO search_object (search_type, namespace, description, "database", table_name, class_name, title, "schema") VALUES ('sthpw/milestone', 'sthpw', 'Project Milestones', 'sthpw', 'milestone', 'pyasm.biz.Milestone', 'Project Milestones', 'public');
INSERT INTO search_object (search_type, namespace, description, "database", table_name, class_name, title, "schema") VALUES ('sthpw/schema', 'sthpw', 'Schema', 'sthpw', 'schema', 'pyasm.biz.Schema', 'Schema', 'public');



INSERT INTO search_object (search_type, namespace, description, "database", table_name, class_name, title, "schema") VALUES ('sthpw/command_log', 'sthpw', 'Historical log of all of the commands executed', 'sthpw', 'command_log', 'pyasm.command.CommandLog', 'Command Log', 'public');
INSERT INTO search_object (search_type, namespace, description, "database", table_name, class_name, title, "schema") VALUES ('sthpw/file', 'sthpw', 'A record of all files that are tracked', 'sthpw', 'file', 'pyasm.biz.file.File', 'File', 'public');
INSERT INTO search_object (search_type, namespace, description, "database", table_name, class_name, title, "schema") VALUES ('sthpw/login_group', 'sthpw', 'List of groups that user belong to', 'sthpw', 'login_group', 'pyasm.security.LoginGroup', 'Groups', 'public');
INSERT INTO search_object (search_type, namespace, description, "database", table_name, class_name, title, "schema") VALUES ('sthpw/note', 'sthpw', 'Notes', 'sthpw', 'note', 'pyasm.biz.Note', 'Notes', NULL);
INSERT INTO search_object (search_type, namespace, description, "database", table_name, class_name, title, "schema") VALUES ('sthpw/pipeline', 'sthpw', 'List of piplines available for sobjects', 'sthpw', 'pipeline', 'pyasm.biz.Pipeline', 'Pipelines', 'public');
INSERT INTO search_object (search_type, namespace, description, "database", table_name, class_name, title, "schema") VALUES ('sthpw/status_log', 'sthpw', 'Log of status changes', 'sthpw', 'status_log', 'pyasm.search.SObject', 'Status Log', 'public');
INSERT INTO search_object (search_type, namespace, description, "database", table_name, class_name, title, "schema") VALUES ('sthpw/notification', 'sthpw', 'Different types of Notification', 'sthpw', 'notification', 'pyasm.biz.Notification', 'Notification', 'public');
INSERT INTO search_object (search_type, namespace, description, "database", table_name, class_name, title, "schema") VALUES ('sthpw/group_notification', 'sthpw', 'Associate one of more kinds of notification with groups', 'sthpw', 'group_notification', 'pyasm.biz.GroupNotification', 'Group Notification', 'public');
INSERT INTO search_object (search_type, namespace, description, "database", table_name, class_name, title, "schema") VALUES ('sthpw/snapshot', 'sthpw', 'All versions of snapshots of assets', 'sthpw', 'snapshot', 'pyasm.biz.Snapshot', 'Snapshot', 'public');
INSERT INTO search_object (search_type, namespace, description, "database", table_name, class_name, title, "schema") VALUES ('sthpw/ticket', 'sthpw', 'Valid login tickets to enter the system', 'sthpw', 'ticket', 'pyasm.security.Ticket', 'Ticket', 'public');
INSERT INTO search_object (search_type, namespace, description, "database", table_name, class_name, title, "schema") VALUES ('sthpw/search_object', 'sthpw', 'List of all the search objects', 'sthpw', 'search_object', 'pyasm.search.SearchType', 'Search Objects', 'public');
INSERT INTO search_object (search_type, namespace, description, "database", table_name, class_name, title, "schema") VALUES ('sthpw/wdg_settings', 'sthpw', 'Persistent store for widgets to remember user settings', 'sthpw', 'wdg_settings', 'pyasm.web.WidgetSettings', 'Widget Settings', 'public');
INSERT INTO search_object (search_type, namespace, description, "database", table_name, class_name, title, "schema") VALUES ('sthpw/transaction_log', 'sthpw', NULL, 'sthpw', 'transaction_log', 'pyasm.search.TransactionLog', 'Transaction Log', 'public');
INSERT INTO search_object (search_type, namespace, description, "database", table_name, class_name, title, "schema") VALUES ('sthpw/trigger_in_command', 'sthpw', 'Triggers contained in Command', 'sthpw', 'trigger_in_command', 'pyasm.biz.TriggerInCommand', 'Command Triggers', 'public');
INSERT INTO search_object (search_type, namespace, description, "database", table_name, class_name, title, "schema") VALUES ('sthpw/sobject_log', 'sthpw', 'Log of actions on an sobject', 'sthpw', 'sobject_log', 'pyasm.search.SObject', 'SObject Log', 'public');
INSERT INTO search_object (search_type, namespace, description, "database", table_name, class_name, title, "schema") VALUES ('sthpw/project_type', 'sthpw', 'Project Type', 'sthpw', 'project_type', 'pyasm.biz.ProjectType', 'Project Type', 'public');
INSERT INTO search_object (search_type, namespace, description, "database", table_name, class_name, title, "schema") VALUES ('sthpw/pref_setting', 'sthpw', 'Preference Setting', 'sthpw', '{public}.pref_setting', 'pyasm.biz.PrefSetting', 'Pref Setting', 'public');
INSERT INTO search_object (search_type, namespace, description, "database", table_name, class_name, title, "schema") VALUES ('sthpw/access_rule', 'sthpw', 'Access Rules', 'sthpw', '{public}.access_rule', 'pyasm.security.AccessRule', 'Access Rule', 'public');
INSERT INTO search_object (search_type, namespace, description, "database", table_name, class_name, title, "schema") VALUES ('sthpw/access_rule_in_group', 'sthpw', 'Access Rules In Group', 'sthpw', '{public}.access_rule_in_group', 'pyasm.security.AccessRuleInGroup', '', 'public');
INSERT INTO search_object (search_type, namespace, description, "database", table_name, class_name, title, "schema") VALUES ('sthpw/clipboard', 'sthpw', 'Clipboard', 'sthpw', '{public}.clipboard', 'pyasm.biz.Clipboard', '', 'public');
INSERT INTO search_object (search_type, namespace, description, "database", table_name, class_name, title, "schema") VALUES ('sthpw/pref_list', 'sthpw', 'Preferences List', 'sthpw', '{public}.pref_list', 'pyasm.biz.PrefList', '', 'public');
INSERT INTO search_object (search_type, namespace, description, "database", table_name, class_name, title, "schema") VALUES ('sthpw/translation', 'sthpw', 'Locale Translations', 'sthpw', '{public}.translation', 'pyasm.search.SObject', '', 'public');
INSERT INTO search_object (search_type, namespace, description, "database", table_name, class_name, title, "schema") VALUES ('sthpw/notification_log', 'sthpw', 'Notification Log', 'sthpw', '{public}.notification_log', 'pyasm.search.SObject', 'Notification Log', 'public');
INSERT INTO search_object (search_type, namespace, description, "database", table_name, class_name, title, "schema") VALUES ('sthpw/notification_login', 'sthpw', 'Notification Login', 'sthpw', '{public}.notification_login', 'pyasm.search.SObject', '', 'public');
INSERT INTO search_object (search_type, namespace, description, "database", table_name, class_name, title, "schema") VALUES ('sthpw/timecard', 'sthpw', 'Timecard Registration', 'sthpw', 'timecard', 'pyasm.search.SObject', 'Timecard', NULL);
INSERT INTO search_object (search_type, namespace, description, "database", table_name, class_name, title, "schema") VALUES ('sthpw/connection', 'sthpw', 'Connections', 'sthpw', 'connection', 'pyasm.biz.SObjectConnection', 'Connections', NULL);




INSERT INTO search_object (search_type, namespace, description, "database", table_name, class_name, title, "schema") VALUES ('sthpw/remote_repo', 'sthpw', 'Remote Repositories', 'sthpw', 'remote_repo', 'pyasm.biz.RemoteRepo', 'Remote Repositories', NULL);
INSERT INTO search_object (search_type, namespace, description, "database", table_name, class_name, title, "schema") VALUES ('sthpw/widget_extend', 'sthpw', 'Extend Widget', 'sthpw', 'widget_extend', 'pyasm.search.SObject', 'widget_extend', 'public');
INSERT INTO search_object (search_type, namespace, description, "database", table_name, class_name, title, "schema") VALUES ('sthpw/search_type', 'sthpw', 'List of all the search objects', 'sthpw', 'search_object', 'pyasm.search.SearchType', 'Search Objects', 'public');
INSERT INTO search_object (search_type, namespace, description, "database", table_name, class_name, title, "schema") VALUES ('sthpw/snapshot_type', 'sthpw', 'Snapshot Type', 'sthpw', 'snapshot_type', 'pyasm.biz.SnapshotType', 'Snapshot Type', 'public');
INSERT INTO search_object (search_type, namespace, description, "database", table_name, class_name, title, "schema") VALUES ('sthpw/widget_config', 'sthpw', 'Widget Config Data', 'sthpw', 'widget_config', 'pyasm.search.WidgetDbConfig', 'Widget Config Data', 'public');
INSERT INTO search_object (search_type, namespace, description, "database", table_name, class_name, title, "schema") VALUES ('sthpw/debug_log', 'sthpw', 'Debug Log', 'sthpw', 'debug_log', 'pyasm.biz.DebugLog', 'Debug Log', 'public');
INSERT INTO search_object (search_type, namespace, description, "database", table_name, class_name, title, "schema") VALUES ('unittest/person', 'unittest', 'Unittest Person', 'unittest', 'person', 'pyasm.search.SObject', 'Unittest Person', 'public');
INSERT INTO search_object (search_type, namespace, description, "database", table_name, class_name, title, "schema") VALUES ('unittest/city', 'unittest', 'Unittest City', 'unittest', 'city', 'pyasm.search.SObject', 'Unittest City', 'public');
INSERT INTO search_object (search_type, namespace, description, "database", table_name, class_name, title, "schema") VALUES ('unittest/country', 'unittest', 'Unittest Country', 'unittest', 'country', 'pyasm.search.SObject', 'Unittest Country', 'public');
INSERT INTO search_object (search_type, namespace, description, "database", table_name, class_name, title, "schema") VALUES ('sthpw/login', 'sthpw', 'List of users', 'sthpw', 'login', 'pyasm.security.Login', 'Users', 'public');
INSERT INTO search_object (search_type, namespace, description, "database", table_name, class_name, title, "schema") VALUES ('sthpw/trigger', 'sthpw', 'Triggers', 'sthpw', 'trigger', 'pyasm.biz.TriggerSObj', 'Triggers', 'public');



INSERT INTO search_object (search_type, namespace, description, "database", table_name, class_name, title, "schema") VALUES ('config/widget_config', 'config', 'Widget Config', '{project}', 'widget_config', 'pyasm.search.WidgetDbConfig', 'Widget Config', 'public');

INSERT INTO search_object (search_type, namespace, description, "database", table_name, class_name, title, "schema") VALUES ('config/custom_script', 'config', 'Custom Script', '{project}', 'custom_script', 'pyasm.search.SObject', 'Custom Script', 'public');

INSERT INTO search_object (search_type, namespace, description, "database", table_name, class_name, title, "schema") VALUES ('config/naming', 'config', 'Naming', '{project}', '{public}.naming', 'pyasm.biz.Naming', '', 'public');

INSERT INTO search_object (search_type, namespace, description, "database", table_name, class_name, title, "schema") VALUES ('sthpw/cache', 'sthpw', 'Cache', 'sthpw', '{public}.cache', 'pyasm.search.SObject', '', 'public');

INSERT INTO search_object (search_type, namespace, description, "database", table_name, class_name, title, "schema") VALUES ('config/prod_setting', 'config', 'Production Settings', '{project}', 'prod_setting', 'pyasm.prod.biz.ProdSetting', 'Production Settings', 'public');

INSERT INTO "search_object" ("search_type", "namespace", "description", "database", "table_name", "class_name", "title", "schema") VALUES ('config/client_trigger', 'config', 'Client Trigger', '{project}', 'spt_client_trigger', 'pyasm.search.SObject', 'Client Trigger', 'public'); 

INSERT INTO "search_object" ("search_type", "namespace", "description", "database", "table_name", "class_name", "title", "schema") VALUES ('config/url', 'config', 'Custom URL', '{project}', 'spt_url', 'pyasm.search.SObject', 'Custom URL', 'public'); 


INSERT INTO "search_object" ("search_type", "namespace", "description", "database", "table_name", "class_name", "title", "schema") VALUES ('config/plugin', 'config', 'Plugin', '{project}', 'spt_plugin', 'pyasm.search.SObject', 'Plugin', 'public'); 

INSERT INTO "search_object" ("search_type", "namespace", "description", "database", "table_name", "class_name", "title", "schema") VALUES ('config/trigger', 'config', 'Triggers', '{project}', 'spt_trigger', 'pyasm.biz.TriggerSObj', 'Triggers', 'public');

INSERT INTO "search_object" ("search_type", "namespace", "description", "database", "table_name", "class_name", "title", "schema") VALUES ('config/process', 'config', 'Processes', '{project}', 'spt_process', 'pyasm.search.SObject', 'Processes', 'public');

INSERT INTO "search_object" ("search_type", "namespace", "description", "database", "table_name", "class_name", "title", "schema") VALUES ('sthpw/work_hour', 'sthpw', 'Work Hours', 'sthpw', 'work_hour', 'pyasm.biz.WorkHour', 'Work Hours', 'public');

INSERT INTO "search_object" ("search_type", "namespace", "description", "database", "table_name", "class_name", "title", "schema") VALUES ('sthpw/sobject_list', 'sthpw', 'SObject List', 'sthpw', 'sobject_list', 'pyasm.search.SObject', 'SObjectList', 'public');
