###########################################################
#
# Copyright (c) 2010, Southpaw Technology
#                     All Rights Reserved
#
# PROPRIETARY INFORMATION.  This software is proprietary to
# Southpaw Technology, and is not to be reproduced, transmitted,
# or disclosed in any way without written permission.
#
#

__all__ = ['ProjectTemplateWdg', 'ProjectTemplateEditWdg', 'ProjectTemplateRemoveCmd', 'ProjectTemplateInstallCmd']

from pyasm.common import Environment, TacticException, Config
from pyasm.command import Command
from pyasm.web import DivWdg, Table
from pyasm.widget import ButtonWdg, ProdIconButtonWdg, TextWdg, TextAreaWdg, CheckboxWdg
from pyasm.search import Search

import os, codecs
import zipfile, shutil

from tactic.ui.common import BaseRefreshWdg
from tactic.ui.container import ResizableTableWdg
from tactic.ui.widget import ActionButtonWdg


class ProjectTemplateWdg(BaseRefreshWdg):

    def get_display(my):

        div = DivWdg()
        div.add_class("spt_project_template_top")
        my.set_as_panel(div)

        div.add_color("background", "background")

        upload_div = DivWdg()
        upload_div.add_style("padding: 10px")
        upload_div.add_style("width: 600px")



        # add the main layout
        table = ResizableTableWdg()
        table.add_color("color", "color")
        div.add(table)

        table.add_row()
        left = table.add_cell()
        left.add_border()
        left.add_style("min-width: 150px")
        left.add_style("height: 400px")

        left.add(my.get_templates_wdg() )

        right = table.add_cell()
        right.add_border()
        right.add_style("width: 400px")
        right.add_style("height: 400px")
        right.add_style("padding: 5px")
        right.add_class("spt_project_template_content")


        empty_div = DivWdg()
        empty_div.add_style("margin: 40px")
        empty_div.add_style("width: 300px")
        empty_div.add_style("height: 150px")
        empty_div.add_border()
        empty_div.add_color("background", "background3")

        right.add(empty_div)

        return div



    def get_templates_wdg(my):

        div = DivWdg()
        div.add_style("padding: 5px")
        div.add_color("background", "background3")
        div.add_style("height: 100%")

        title_div = DivWdg()
        div.add(title_div)
        title_div.add("Installed Templates")
        title_div.add_gradient("background", "background", 0, -10)
        title_div.add_color("color", "color3")
        title_div.add_style("margin: -5px -5px 5px -5px")
        title_div.add_style("font-weight: bold")
        title_div.add_style("padding: 5px")

        templates_div = DivWdg()
        div.add(templates_div)

        templates_div.add_relay_behavior( {
            'type': 'mouseup',
            'bvr_match_class': 'spt_template',
            'cbjs_action': '''
            var top = bvr.src_el.getParent(".spt_project_template_top");
            var content = top.getElement(".spt_project_template_content")
            var class_name = 'tactic.ui.app.ProjectTemplateEditWdg';
            var template_dir = bvr.src_el.getAttribute("spt_template_dir");
            var kwargs = {
                template_dir: template_dir
            };
            spt.panel.load(content, class_name, kwargs);
            '''
        } )
        templates_div.add_class("hand")

        template_dir = Environment.get_template_dir()
        if not template_dir:
            raise Exception("No template dir defined")
        #template_dir = "/home/apache/project_templates"
        dirnames = os.listdir(template_dir)

        #templates = ['scrum', 'tickets', 'vfx', 'game']
        for template in dirnames:

            path = "%s/%s" % (template_dir, template)

            if not os.path.isdir(path):
                continue

            template_div = DivWdg()
            templates_div.add(template_div)
            template_div.add(template)
            template_div.add_style("padding: 3px")
            template_div.add_class("spt_template")
            template_div.add_attr("spt_template_dir", path)

        return div



class ProjectTemplateEditWdg(BaseRefreshWdg):
    def get_display(my):

        top = my.top
        my.set_as_panel(top)
        top.add_style("width: 500px")

        template_dir = my.kwargs.get("template_dir")

        manifest_path = "%s/manifest.xml" % template_dir
        top.add(manifest_path)

        f = open(manifest_path)
        manifest_xml = f.read()
        f.close()


        button = ActionButtonWdg(title="Install", tip="Install template to installation")
        top.add(button)
        button.add_behavior( {
            'type': 'click_up',
            'cbjs_action': '''
            var applet = spt.Applet.get();
            var paths = applet.open_file_browser();
            var server = TacticServerStub.get();
            var cmd = "tactic.ui.app.ProjectTemplateInstallCmd";

            if (!paths.length) {
                return;
            }

            for (var i = 0; i < paths.length; i++) {
                server.upload_file(paths[i]);
            }

            var kwargs = {
                paths: paths
            };
            server.execute_cmd(cmd, kwargs);


            var top = bvr.src_el.getParent(".spt_project_template_top");
            spt.panel.refresh(top);

            '''
        } )


        button = ActionButtonWdg(title="Remove", tip="Remove template from installation")
        top.add(button)
        button.add_behavior( {
            'type': 'click_up',
            'cbjs_action': '''
            var cmd = "tactic.ui.app.ProjectTemplateRemoveCmd";
            var server = TacticServerStub.get();

            var template = "vfx_template";
            spt.app_busy.show("Removing Template", template)

            var kwargs = {
                'template': template
            }
            server.execute_cmd(cmd, kwargs);

            var top = bvr.src_el.getParent(".spt_project_template_top");
            spt.panel.refresh(top);


            spt.app_busy.hide();
            '''
        } )

        return top


class ProjectTemplateRemoveCmd(Command):
    def execute(my):
        template = my.kwargs.get("template")

        template_dir = Environment.get_template_dir()

        template = my.kwargs.get("template")


        from pyasm.search import FileUndo
        path = "%s/%s-1.0.0.zip" % (template_dir, template)
        if not os.path.exists(path):
            raise Exception("Path [%s] does not exists" % path)

        FileUndo.remove(path)

        path = "%s/%s" % (template_dir, template)
        FileUndo.remove(path)


class ProjectTemplateInstallCmd(Command):

    def execute(my):

        from pyasm.common import ZipUtil
        ziputil = ZipUtil()

        paths = my.kwargs.get("paths")

        upload_dir = Environment.get_upload_dir()
        template_dir = Environment.get_template_dir()

        for path in paths:
            path = path.replace("\\", "/")
            basename = os.path.basename(path)
            upload_path = "%s/%s" % (upload_dir, basename)

            if not upload_path.endswith(".zip"):
                continue

            print "upload: ", upload_path
            if not os.path.exists(upload_path):
                continue


            print "template_dir: ", template_dir
            shutil.move(upload_path, template_dir)
            to_path = "%s/%s" % (template_dir, basename)

            # unzip the file
            ziputil.extract(to_path)







