###########################################################
#
# Copyright (c) 2005-2008, Southpaw Technology
#                     All Rights Reserved
#
# PROPRIETARY INFORMATION.  This software is proprietary to
# Southpaw Technology, and is not to be reproduced, transmitted,
# or disclosed in any way without written permission.
#
#
#

__all__ = ['CustomLayoutEditWdg', 'CustomLayoutEditTestWdg', 'CustomLayoutEditSaveCmd']

from pyasm.search import Search, SearchType
from pyasm.web import DivWdg, Table
from pyasm.widget import IconWdg
from pyasm.widget import TextWdg, TextAreaWdg
from pyasm.command import Command
from tactic.ui.common import BaseRefreshWdg

from tactic.ui.widget import ButtonRowWdg, ButtonNewWdg, ActionButtonWdg, SwapDisplayWdg


class CustomLayoutEditWdg(BaseRefreshWdg):



    def get_title_wdg(my, title, content_id=None, is_on=True):
        title_wdg = DivWdg()

        title = "<b>%s:</b>" % title
        swap = SwapDisplayWdg(title=title, is_on=is_on)
        title_wdg.add(swap)
        if content_id:
            swap.set_content_id(content_id)

        title_wdg.add_gradient("background", "background", -10)
        title_wdg.add_style("padding: 5px")
        title_wdg.add_border()


        return title_wdg




    def get_display(my):

        top = my.top
        top.add_class("spt_custom_layout_top")
        top.add_style("padding: 20px")
        top.add_color("color", "color")
        top.add_color("background", "background")
        my.set_as_panel(top)

        inner = DivWdg()
        top.add(inner)

        table = Table()
        inner.add(table)
        table.add_row()

        search = Search("config/widget_config")
        search.add_filter("search_type", "CustomLayoutWdg")
        search.add_order_by("view")
        configs = search.get_sobjects()


        view = my.kwargs.get("view")
        if view == '__new__':
            cur_config = None
        if view:
            search = Search("config/widget_config")
            search.add_filter("search_type", "CustomLayoutWdg")
            search.add_filter("view", view)
            cur_config = search.get_sobject()
        else:
            view = ""
            cur_config = None


        tr, shelf = table.add_row_cell()
        shelf_wdg = DivWdg()
        shelf.add(shelf_wdg)
        shelf_wdg.add_style("height: 30px")

        shelf_wdg.add_style("height: 35px")
        shelf_wdg.add_color("background", "background", -10)
        shelf_wdg.add_style("padding: 3px")
        shelf_wdg.add_style("margin: -2px -2px -1px -2px")
        shelf_wdg.add_border()


        # refresh button
        button_row = ButtonRowWdg()
        shelf_wdg.add(button_row)
        button_row.add_style("float: left")
        button = ButtonNewWdg(title="Refresh", icon=IconWdg.REFRESH)
        button_row.add(button)

        button.add_behavior( {
            'type': 'click_up',
            'cbjs_action': '''
            spt.app_busy.show("Refreshing ...")
            var top = bvr.src_el.getParent(".spt_custom_layout_top");
            spt.panel.refresh(top);
            spt.app_busy.hide();
            '''
        } )


        # add new button
        button = ButtonNewWdg(title="Add New", icon=IconWdg.ADD)
        button_row.add(button)


        button.add_behavior( {
            'type': 'click_up',
            'cbjs_action': '''
            var top = bvr.src_el.getParent(".spt_custom_layout_top");
            top.setAttribute("spt_view", "__new__");
            spt.panel.refresh(top);
            '''
        } )


        button = ButtonNewWdg(title="Save", icon=IconWdg.SAVE)
        button_row.add(button)

        button.add_behavior( {
            'type': 'click_up',
            'cbjs_action': '''
            var top = bvr.src_el.getParent(".spt_custom_layout_top");
            var values = spt.api.Utility.get_input_values(top, null, false);

            var cmd = 'tactic.ui.tools.CustomLayoutEditSaveCmd';
            var html = values.html;
            var style = values.style;
            var view = values.view;

            if (!view) {
                spt.alert("A view name must be provided to save");
                return;
            }

            spt.app_busy.show("Saving ["+view+"] ...")

            var kwargs = {
                view: view,
                html: html,
                style: style

            }
            spt.app_busy.show("Saving Custom Layout");
            var server = TacticServerStub.get();
            server.execute_cmd(cmd, kwargs);
            spt.app_busy.hide();

            top.setAttribute("spt_view", view);
            spt.panel.refresh(top);

            '''
        } )



        button = ButtonNewWdg(title="Test", icon=IconWdg.ARROW_RIGHT)
        button_row.add(button)

        button.add_behavior( {
            'type': 'click_up',
            'cbjs_action': '''
            var top = bvr.src_el.getParent(".spt_custom_layout_top");
            var values = spt.api.Utility.get_input_values(top, null, false);

            var class_name = 'tactic.ui.tools.CustomLayoutEditTestWdg';
            spt.panel.load_popup("Test Custom Layout", class_name, values);

        ''' } )


        table.add_row()

        table.add_border()


        left = table.add_cell()
        left.add_color("background", "background3")

        left_div = DivWdg()
        left.add(left_div)
        left.add_style("vertical-align: top")
        left_div.add_style("width: 250px")
        left_div.add_color("color", "color3")
        left_div.add_color("height", "100%")

        left_div.add_style("padding-bottom: 15px")
        left_div.set_unique_id()
        left_div.add_smart_styles( "spt_custom_layout_item", {
            'padding': '3px 6px 3px 10px'
        } )
        left_div.add_relay_behavior( { 
            'type': 'mouseup',
            'bvr_match_class': 'spt_custom_layout_item',
            'cbjs_action': '''
            var top = bvr.src_el.getParent(".spt_custom_layout_top");
            var content = top.getElement(".spt_custom_layout_content");
            var view = bvr.src_el.getAttribute("spt_view");

            top.setAttribute("spt_view", view)
            spt.panel.refresh(top);
            '''
        } )


        bg_color = left_div.get_color("background3")
        bg_color2 = left_div.get_color("background3", -10)

        left_div.add_relay_behavior( {
            'type': 'mouseover',
            'bg_color': bg_color2,
            'bvr_match_class': 'spt_custom_layout_item',
            'cbjs_action': '''
            bvr.src_el.setStyle("background-color", bvr.bg_color);
            '''
        } )

        left_div.add_relay_behavior( {
            'type': 'mouseout',
            'bg_color': bg_color2,
            'bvr_match_class': 'spt_custom_layout_item',
            'cbjs_action': '''
            bvr.src_el.setStyle("background-color", "");
            '''
        } )



        title_wdg = DivWdg()
        left_div.add(title_wdg)
        title_wdg.add("<b>Custom Layout Views</b>")
        title_wdg.add_color("color", "color3")
        title_wdg.add_gradient("background", "background3", -5)
        title_wdg.add_style("padding: 5px")
        title_wdg.add_style("margin-bottom: 10px")


        for config in configs:
            config_div = DivWdg()
            left_div.add(config_div)
            config_div.add_class("spt_custom_layout_item")
            config_view = config.get_value("view")
            config_div.add(config_view)
            config_div.add_attr("spt_view", config_view)

            config_div.add_class("hand")


        right = table.add_cell()
        right.add_style("vertical-align: top")
        right_div = DivWdg()
        right.add(right_div)
        right_div.add_color("color", "color")
        right_div.add_color("background", "background", -3)
        right_div.add_style("min-width: 600px")
        right_div.add_style("min-height: 400px")
        right_div.add_style("height: 100%x")
        right_div.add_class("spt_custom_layout_content")

        right_div.add_style("padding: 20px")

        if view != '__new__' and not cur_config:
            nothing_wdg = DivWdg()
            right_div.add(nothing_wdg)
            nothing_wdg.add_border()
            nothing_wdg.add("<b>No Layout Selected</b>")
            nothing_wdg.add_style("padding: 30px")
            nothing_wdg.add_color("color", "color3")
            nothing_wdg.add_color("background", "background3")
            nothing_wdg.add_style("width: 300px")
            nothing_wdg.add_style("text-align: center")
            nothing_wdg.add_style("margin: 60px auto")

        else:

            if view == '__new__':
                html = ''
                style = ''
                behavior_nodes = []
            else:
                xml = cur_config.get_xml_value("config")

                style_node = xml.get_value("config/%s/html/style" % view)
                style = style_node

                behavior_nodes = xml.get_nodes("config/%s/behavior" % view)

                html_nodes = xml.get_nodes("config/%s/html/*" % view)
                html = ''
                for node in html_nodes:
                    if xml.get_node_name(node) == 'style':
                        continue

                    html = xml.to_string(node)



            text = TextWdg("view")
            content_id = text.set_unique_id()

            title_wdg = my.get_title_wdg("View", content_id)
            right_div.add(title_wdg)

            right_div.add(text)
            text.add_class("spt_view")
            text.add_style("width: 100%")
            text.add_style("font-size: 12px")
            text.add_style("font-family: courier")
            if view != '__new__':
                text.set_value(view)

            right_div.add("<br/>"*2)


            # html
            text = TextAreaWdg("html")
            content_id = text.set_unique_id()

            title_wdg = my.get_title_wdg("HTML", content_id)
            right_div.add(title_wdg)

            right_div.add(text)
            text.add_style("width: 100%")
            text.add_style("height: 300px")
            text.add_style("min-height: 300px")
            text.add_style("font-size: 12px")
            text.add_style("font-family: courier")
            if html:
                text.set_value(html)


            right_div.add("<br/>"*2)

            # styles
            text = TextAreaWdg("style")
            content_id = text.set_unique_id()
            text.add_style("display: none")

            title_wdg = my.get_title_wdg("Styles", content_id, is_on=False)
            right_div.add(title_wdg)

            right_div.add(text)
            text.add_style("width: 100%")
            text.add_style("height: 300px")
            text.add_style("min-height: 300px")
            text.add_style("font-size: 12px")
            text.add_style("font-family: courier")
            text.set_value(style)



            right_div.add("<br/>"*2)

            # behaviors
            text = TextAreaWdg("behavior")
            content_id = text.set_unique_id()
            text.add_style("display: none")

            title_wdg = my.get_title_wdg("Behaviors", content_id, is_on=False)
            right_div.add(title_wdg)

            right_div.add(text)
            text.add_style("width: 100%")
            text.add_style("height: 300px")
            text.add_style("min-height: 300px")
            text.add_style("font-size: 12px")
            text.add_style("font-family: courier")

            xx = []
            for behavior_node in behavior_nodes:
                xx.append( xml.to_string(behavior_node))
            behavior_str = "\n\n".join(xx)

            text.set_value(behavior_str)







        if my.kwargs.get("is_refresh") == 'true':
            return inner
        else:
            return top


class CustomLayoutEditTestWdg(BaseRefreshWdg):

    def get_display(my):

        html = my.kwargs.get("html")
        style = my.kwargs.get("style")
        view = my.kwargs.get("view")
        behavior = my.kwargs.get("behavior")

        config_xml = CustomLayoutEditSaveCmd.build_xml(view, html, style, behavior)

        top = my.top
        top.add_style("padding: 20px")
        top.add_border()
        top.add_color("color", "color")
        top.add_color("background", "background")

        view = my.kwargs.get("view")


        from tactic.ui.panel import CustomLayoutWdg
        layout = CustomLayoutWdg(config_xml=config_xml, view=view, include_mako=True)
        #layout = CustomLayoutWdg(view=view, include_mako=True)
        top.add(layout)


        return top



class CustomLayoutEditSaveCmd(Command):

    def build_xml(cls, view, html, style=None, behavior=None):

        # build up the custom layout
        html = html.strip()
        html = html.replace("<%", "<![CDATA[\n<%")
        html = html.replace("%>", "%> ]]>")

        if style:
            style = style.strip()

        layout = []
        layout.append("<config>")
        layout.append("<%s>" % view)
        layout.append("<html>")


        if style:
            layout.append('''<style type="text/css">''')
            layout.append(style)
            layout.append('''</style>''')
        if html:
            #layout.append('''<div class="spt_top">''')
            layout.append(html)
            #layout.append('''</div>''')




        layout.append("</html>")

        # FIXME: this is very tenuous
        if behavior:
            import re
            behavior = behavior.strip()

            p = re.compile("(<behavior.*?>)")
            behavior = p.sub("\\1<![CDATA[", behavior)
            behavior = behavior.replace("</behavior>", "]]></behavior>")

            layout.append(behavior)


        layout.append("</%s>" % view)
        layout.append("</config>")

        config_xml = "\n".join(layout)



        #f = open("/tmp/tt.xml", 'w')
        #f.write(config_xml)
        #f.close()

        #f = open("/tmp/tt.xml")
        #config_xml = f.read()
        #f.close()

        return config_xml

    build_xml = classmethod(build_xml)



    def execute(my):

        html = my.kwargs.get("html")
        style = my.kwargs.get("style")
        view = my.kwargs.get("view")
        behavior = my.kwargs.get("behavior")

        config_xml = my.build_xml(view, html, style, behavior)


        search = Search("config/widget_config")
        search.add_filter("search_type", "CustomLayoutWdg")
        search.add_filter("view", view)
        sobject = search.get_sobject()
        if not sobject:
            sobject = SearchType.create("config/widget_config")
            sobject.set_value("search_type", "CustomLayoutWdg")
            sobject.set_value("view", view)

        sobject.set_value("config", config_xml)
        sobject.commit()






